%  [grow_rates,kmat,mshpts] =
%        lambda(dat,numclass,fit_methods,reg_order,makepics,mshsize)
%
%
% This function calculates the growth rate for the population
% data set.  It calls grow.m and fecund.m to fit the components
% of the kernel.
%
%  input:  dat   -- matrix with the data
%
%  optional input:   If these inputs are not given, default values will be set
%          numclass  --  default = 5  -- size of matrix model to fit 
%
%          fit_methods  --  default = [2 2 2 2 2 2] 
%                     How each part of the integral model is fit
%                     1 for spline
%                     2 for regression
%
%                     Six functions are fit to form the kernel
%                        1.  survival vs. size
%                        2.  new size vs. old size
%                        3.  variance of new size vs. old size. 
%                        4.  #offspring vs. size
%                        5.  mean offpring size vs. parent size
%                        6.  variance of offspring size vs. parent size
% 
%                     NOTE: the spline option uses a cubic smoothing spline 
%                           and it requires the spline toolbox in Matlab.
%
%          reg_order --  default = [1 1 1 1 1 1]   --  fits any regressions 
%                        with 1st order change the 1s to 2, 3, etc for 
%                        higher order fits
%
%          makepics  --  default = 1   --  plot the data and data fits.  
%                        Any other value will supress plots
%    
%          mshsize   --  default = 150  --  # of mesh points at which kernel
%                                             value is calculate
%
%  return: grow_rates -- 3 values of the growth rate
%              1.  from the integral model
%              2.  from the data (# @ time t+1 / # @ time t)
%              3.  from a matrix with a user definded number of classes
%              grow_rates = [ilam dlam mlam]
%
%          kmat  --  matrix of kernel values of size mshsize x mshsize
%
%          mshpts  -- size values (x1, x2, ... xn) corresponding to kmat values
% 
% [grow_rates,kmat,mshpts] = 
%         lambda(dat,numclass,fit_methods,reg_order,makepics,mshsize)
%

function [grow_rates, kmat, mshpts] = lambda(dat,numclass,fit_method,reg_order,makepics,mshsize)


global survpp growpp vgrowpp cut mkpp vkpp ksizpp vsizpp
global mshmin mshmax
global lowval hival
global grofit fecfit 
global surv_order grow_order vargrow_order 
global num_off_order offsize_order offvar_order

if nargin < 1
   error('  You must enter a data set name')
end

if nargin < 2 | isempty(numclass)
   % Set number of stage classes in the fitted matrix mode to 5
   numclass = 5;
	disp('     Using default (5) stages for a matrix model')
end


if nargin < 3 | isempty(fit_method) | length(fit_method) ~= 6
   % Set default of all fits to regression
   fit_method = [1 2 2 2 2 2];
   disp('     Using default values for fitting')
end

surv_fit_method = fit_method(1);
grow_fit_method = fit_method(2);
vargrow_fit_method = fit_method(3);
num_off_fit_method = fit_method(4);
offsize_fit_method = fit_method(5);
offvar_fit_method = fit_method(6);

if nargin < 4 | isempty(reg_order) | length(reg_order) ~= 6
   %  Set the default regression order to 1 for all regression fits
   reg_order = [1 1 1 1 1 1];
   disp('     Using default values for regression order')
end

surv_order = reg_order(1);
grow_order = reg_order(2);
vargrow_order = reg_order(3);
num_off_order = reg_order(4);
offsize_order = reg_order(5);
offvar_order = reg_order(6);

if nargin < 5 | isempty(makepics)
   % Set the default to make plots of the data & fits
   makepics = 1;
end

dopics = makepics;


if nargin < 6 | isempty(mshsize)
   % set default value of 150 for the mesh size
   mshsize = 150;
end

%*************USER SHOULD NOT EDIT BEYOND THIS LINE*******************


grofit = [surv_fit_method grow_fit_method vargrow_fit_method];
fecfit = [num_off_fit_method offsize_fit_method offvar_fit_method];

%  Fit the parts of the kernel
[survpp, growpp, vgrowpp] = grow(dat,dopics,grofit);
[cut, mkpp, ksizpp, vsizpp] = fecund(dat,dopics,fecfit);
vals = [min(dat(:,1)), max(dat(:,1))];
lowval = vals(1);
hival = vals(2);


if grofit(3) == 1
   sigs = sqrt(max(0.000001,fnval(vgrowpp, vals)));
else
   sigs = sqrt(max(0.000001,(vgrowpp(1) + vgrowpp(2).*vals)));
end

mshmin = vals(1) - 3.*sigs(1);
mshmax = vals(2) + 3.*sigs(2);

% - - - - Solve the integral equation for lambda - - - - - -
[kmat, mshpts] = kernmat(mshsize, mshmin, mshmax);
[ilam, lammeth] = intlam(kmat);
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 


% - - - - Calculate lambda from the data #t+1 / # t - - - - -y
dlam = datlam(dat);
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


% - - - - Fit a matrix and calculate lambda - - - - - - - - -
mlam = matlam(dat,numclass);
% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

grow_rates = [ilam, dlam, mlam];




